/* Copyright 2010 Greg Alexander.  See the file COPYING for terms */
/* $Id: eval.c,v 1.41 2010-02-08 03:17:28 greg Exp $ */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "misc.h"
#include "lang.h"
#include "snd.h"

/* this used to be a full-on interprative evaluator, but now it is just for
 * handling the constant arguments to a compiled func */
static value_t
eval(struct tree_rec *tr)
{
	switch(tr->kind) {
		case op_number:
			return tr->u.number;
		case op_string:
			return tr->u.string;
		default:
			assert_str(0,"non-constant in eval");
	}
	return -1;
}


int
do_fn_call(struct sound_rec *s, int *buf, int nents)
{
	value_t a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0;
	struct tree_rec *tr;
	struct compiled_rec *comp;

	tr = s->tr;
	assert_str(tr && tr->kind == op_fn_call, "non-function call in sounds?");
	comp = tr->u.fn_call.func->compiled;

	assert_str(comp != NULL, "only compiled calls allowed here");
	if (tr->u.fn_call.num_args >= 1)
		a0 = eval(tr->u.fn_call.args[0]);
	if (tr->u.fn_call.num_args >= 2)
		a1 = eval(tr->u.fn_call.args[1]);
	if (tr->u.fn_call.num_args >= 3)
		a2 = eval(tr->u.fn_call.args[2]);
	if (tr->u.fn_call.num_args >= 4)
		a3 = eval(tr->u.fn_call.args[3]);
	if (tr->u.fn_call.num_args >= 5)
		a4 = eval(tr->u.fn_call.args[4]);
	if (tr->u.fn_call.num_args >= 6)
		a5 = eval(tr->u.fn_call.args[5]);
	if (tr->u.fn_call.num_args >= 7)
		a6 = eval(tr->u.fn_call.args[6]);
	if (tr->u.fn_call.num_args >= 8)
		a7 = eval(tr->u.fn_call.args[7]);

		/* default to doneness of arg0 */
	return comp->func(curr_sound,
		s->tmps, s->itmps, a0,a1,a2,a3,a4,a5,a6,a7,buf,nents);
}

/* consumes a tree that has been scanned, doing whatever is necessary to
 * "make it so" */
void
do_tree(struct tree_rec *tr)
{
	if (num_errors == 0 && tr && tr->kind == op_fn_call) {
		struct symbol *s;
		value_t *tmps;
		intval_t *itmps;

		s = tr->u.fn_call.func;
		if (!s || !s->defn) {
			do_error("call of undefined func \"%s\"", s->name);
			return;
		}
		if (!s->compiled) {
			s->compiled = compile_func(s->defn);
		}
		if (!s->compiled) {
			do_error("could not compile func \"%s\"", s->name);
			return;
		}
		tmps = safe_malloc(s->compiled->num_temps*sizeof *tmps);
		memset(tmps, 0, s->compiled->num_temps*sizeof *tmps);
		itmps = safe_malloc(s->compiled->num_itemps*sizeof *itmps);
		memset(itmps, 0, s->compiled->num_itemps*sizeof *itmps);
		snd_add(tr, tmps, itmps);
	}
}

