/* Copyright 2010 Greg Alexander.  See the file COPYING for terms */
/* XXX UNUSED XXX */
/* XXX UNUSED XXX */
/* XXX UNUSED XXX */
/* XXX UNUSED XXX */

/* $Id: evt.c,v 1.8 2010-02-08 03:17:28 greg Exp $ */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include "evt.h"
#include "misc.h"

#define BUF_LEN 2048

struct playing_rec {
	struct playing_rec *next;
	char *fn;
	FILE *f;
	char buf[BUF_LEN];
	char *ofs;
	struct timeval when;
};
static struct playing_rec *players = NULL;

static FILE *record_f;
static struct timeval record_tv;


static void player_kill(struct playing_rec *p);


void
evt_list(void)
{
	struct playing_rec *p;
	printf("currently playing:\n");
	for (p = players; p; p = p->next) {
		printf("   %s (%s)\n",p->fn, p->ofs ? p->ofs : p->buf);
	}
	if (record_f) {
		printf("currently recording!\n");
	}
}

/* start playing events */
void
evt_play(char *fn)
{
	struct playing_rec *n;
	FILE *f;
	fn = extended_fn(fn, "evt");
	f = fopen(fn,"r");
	if (!f) {
		perror(fn);
		return;
	}
	n = safe_malloc(sizeof *n);
	n->next = players;
	n->fn = safe_strdup(fn);
	n->f = f;
	n->buf[0] = 0;
	n->ofs = NULL;
	gettimeofday(&(n->when),NULL);
	players = n;
}

/* stop playing events */
void
evt_stop_play(void)
{
	while (players) {
		player_kill(players);
	}
}

/* start recording events */
void
evt_record(char *fn)
{
	fn = extended_fn(fn, "evt");
	record_f = fopen(fn,"w");
	if (!record_f) {
		perror(fn);
		return;
	}
	gettimeofday(&record_tv,NULL);
}

/* stop recording events */
void
evt_stop_record(void)
{
	if (record_f)
		fclose(record_f);
	record_f = NULL;
}

/* process the pending events and figure out how long until the next event */
void
evt_process(struct timeval *tv)
{
	struct playing_rec *p;
	struct timeval now;
	struct timeval when;

	gettimeofday(&now,NULL);
	when.tv_sec = now.tv_sec + 1000;
	when.tv_usec = 0;

restart:
	for (p = players; p; p = p->next) {
		while ((p->when.tv_sec < now.tv_sec) ||
		       ((p->when.tv_sec == now.tv_sec) &&
		        (p->when.tv_usec <= now.tv_usec))) {
			if (p->ofs == NULL) {
				if (!fgets(p->buf,BUF_LEN,p->f)) {
					player_kill(p);
					goto restart;
				}
				p->ofs = p->buf;
				p->buf[strcspn(p->buf,"\r\n")] = 0;
				p->when = now;
			}
			while (isspace(*(p->ofs))) { p->ofs++; }
			if (*(p->ofs) == '~') {
				int ms[3] = {0,0,0};
				int ms_len = 0;
				int sec = 0;
				p->ofs++;
				while (isdigit(*(p->ofs))) {
					sec = (sec*10) + (*(p->ofs)-'0');
					p->ofs++;
				}
				if ((*(p->ofs)) == '.') {
					p->ofs++;
					while (isdigit(*(p->ofs)) && ms_len < 3) {
						ms[ms_len++] = *(p->ofs) - '0';
						p->ofs++;
					}
				}
				p->when.tv_sec += sec;
				p->when.tv_usec += ms[0]*100000 + ms[1]*10000 + ms[2]*1000;
				p->when.tv_sec += p->when.tv_usec / 1000000;
				p->when.tv_usec = p->when.tv_usec % 1000000;
			} else {
				cmd_doit(p->ofs);
				p->ofs = NULL;
			}
		}
		if ((p->when.tv_sec < when.tv_sec) ||
		    ((p->when.tv_sec == when.tv_sec) &&
		     (p->when.tv_usec <= when.tv_usec))) {
			when = p->when;
		}
	}
	tv->tv_sec = when.tv_sec - now.tv_sec;
	tv->tv_usec = (when.tv_usec + 1000000) - now.tv_usec;
	if (tv->tv_usec > 1000000) {
		tv->tv_usec -= 1000000;
	} else {
		tv->tv_sec -= 1;
	}
}

/* an event was generated, record it if necessary */
void
evt_doit(char *line)
{
	struct timeval new_tv;
	long sec,msec;

	if (!record_f)
		return;

	gettimeofday(&new_tv, NULL);
	sec = (new_tv.tv_sec - record_tv.tv_sec);
	msec = ((new_tv.tv_usec + 1000000) - record_tv.tv_usec) / 1000;
	if (msec >= 1000) {
		msec -= 1000;
	} else {
		sec -= 1;
	}
	fprintf(record_f,"~%d.%03d %s\n",(int)sec,(int)msec,line);
	record_tv = new_tv;
}

/* eliminate a playing_rec from the players list */
static void
player_kill(struct playing_rec *p)
{
	struct playing_rec **pp;
	for (pp = &players; *pp && *pp != p; pp = &((*pp)->next))
		;
	if (*pp) {
		*pp = p->next;
		free(p->fn);
		fclose(p->f);
		free(p);
	}
}
