/* Copyright 2010 Greg Alexander.  See the file COPYING for terms */
/* $Id: misc.c,v 1.9 2010-02-08 03:17:28 greg Exp $ */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "misc.h"

/* miscellaneous helper functions */

void *
safe_malloc(size_t len)
{
	void *ptr = malloc(len);
	assert_str((int)ptr,"malloc failed");
	return ptr;
}

char *
safe_strdup(char *s)
{
	int len;
	char *ret;

	if (!s)
		return NULL;

	len = strlen(s) + 1;
	ret = safe_malloc(len);
	memcpy(ret,s,len);
	return ret;
}

void
assert_str(int b,char *s)
{
	if (b)
		return;

	do_cleanup();

	fprintf(stderr,"%s\n",s);
	abort();
}

FILE *
open_search(char *fn, char *mode)
{
	int i;
	static char *efn = NULL;
	static int efn_len = 0;
	int fn_len = strlen(fn);
	FILE *f;

	if (fn[0] == '/') {
		return fopen(fn, mode);
	}

	for (i = 0; i < SEARCH_MAX; i++) {
		int len;
		if (!search_path[i])
			continue;
		len = fn_len + strlen(search_path[i]) + 3;
		if (len >= efn_len) {
			free(efn);
			efn_len = len + 50;
			efn = safe_malloc(efn_len);
		}
		sprintf(efn,"%s/%s",search_path[i],fn);
		f = fopen(efn, mode);
		if (f) return f;
	}
	return NULL;
}

/* Adds ".ext" to fn if there's not already an extension.
 * Returns static buffer. */
char *
extended_fn(char *fn, char *ext)
{
	static char *buf = 0;
	static int buflen = 0;

	if (strchr(fn, '.')) {		/* already extended */
		return fn;
	}

	if (strlen(fn)+strlen(ext)+3 >= buflen) {
		free(buf);
		buflen = strlen(fn) + strlen(ext) + 3;
		buf = safe_malloc(buflen);
	}

	sprintf(buf, "%s.%s", fn, ext);
	return buf;
}
